import { Ref, onMounted, ref } from 'vue'
import { showError } from '../ui'

import type { BasePaginationData } from '@/types/api'

export interface UsePageListOptions<T> {
  query: (
    currentPage: number,
    pageSize: number,
    statusCode?: number,
  ) => Promise<BasePaginationData<T>>
  initPageSize?: number
  initLoad?: boolean
}

export default function usePageList<T>(options: UsePageListOptions<T>) {
  const loading = ref(true)
  const currentPage = ref(1)
  const pageSize = ref(options.initPageSize || 100)
  const total = ref(0)
  const data = ref<T[]>([]) as Ref<T[]>

  const loadData = async () => {
    const { query } = options

    try {
      loading.value = true
      const res = await query(currentPage.value, pageSize.value)
      total.value = res.data.total
      currentPage.value = res.data.current
      pageSize.value = res.data.size
      data.value = res.data.records
    } catch (error) {
      showError(error)
    } finally {
      loading.value = false
    }
  }

  const onCurrentPageChange = (page: number) => {
    currentPage.value = page
    loadData()
  }

  const refresh = () => {
    currentPage.value = 1
    loadData()
  }

  const onPageSizeChange = (newPageSize: number) => {
    pageSize.value = newPageSize
    refresh()
  }

  onMounted(() => {
    if (options.initLoad !== false) {
      refresh()
    }
  })

  return {
    loading,
    currentPage,
    pageSize,
    total,
    data,
    onCurrentPageChange,
    onPageSizeChange,
    refresh,
  }
}
